#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 1A.1 - Histogramme et dictionnaire
# 
# Compter les occurences de nombres dans une liste est assez facile avec un dictionnaire et on peut l'écrire de plusieurs manières différentes.

# In[1]:


import random
num = [random.randint(0,10) for i in range(1000)]


# In[2]:


hist = {}
for i in num:
    hist[i] = hist.get(i, 0) + 1


# In[3]:


hist


# Mesurer le temps que cela prend :

# In[4]:


def count1(num):
    hist = {}
    for i in num:
        hist[i] = hist.get(i, 0) + 1
    return hist

get_ipython().run_line_magic('timeit', 'count1(num)')


# Comparons avec une autre implémentation :

# In[5]:


def count2(num):
    hist = {}
    for i in num:
        if i in hist:
            hist[i] += 1
        else:
            hist[i] = 1
    return hist

get_ipython().run_line_magic('timeit', 'count2(num)')


# Une version plus rapide :

# In[6]:


from collections import Counter

def count3(num):
    return Counter(num)

get_ipython().run_line_magic('timeit', 'count3(num)')


# Une dernière avec [cytoolz](https://toolz.readthedocs.io/en/latest/) :

# In[7]:


from cytoolz import count, groupby
import cytoolz

def count4(num):
    return list(cytoolz.map(lambda x: (x[0], count(x[1])), 
                    groupby(lambda x:x, num).items()))

get_ipython().run_line_magic('timeit', 'count4(num)')


# In[8]: